function [v_rotation, v_wheel, sigma_long, sigma_lat, sigma_res, alpha, beta] = wheel_slip(delta, v_body, psi_dot, omega_wheel, l_front, l_rear, l_tread, r_wheel)

% wheel angular velocity
omega_wheel_fl = omega_wheel(1);
omega_wheel_fr = omega_wheel(2);
omega_wheel_rl = omega_wheel(3);
omega_wheel_rr = omega_wheel(4);

% vehicle speed and velocities
u = v_body(1);
v = v_body(2);
V = sqrt(u^2 + v^2);

% vehicle sideslip angle
beta = atan2(v, u);

% distances from CG to each wheel
r_fl = sqrt(l_front^2 + (l_tread/2)^2);
r_fr = r_fl;
r_rl = sqrt(l_rear^2 + (l_tread/2)^2);
r_rr = r_rl;

% angles from CG to each wheel
theta_fl = atan2(l_tread/2, l_front);
theta_fr = atan2(l_front, l_tread/2);
theta_rl = atan2(l_rear, l_tread/2);
theta_rr = atan2(l_tread/2, l_rear);

% wheel center velocities
v_wheel_fl = [V*cos(beta) - psi_dot*r_fl*sin(theta_fl); V*sin(beta) + psi_dot*r_fl*cos(theta_fl)];
v_wheel_fr = [V*cos(beta) + psi_dot*r_fr*cos(theta_fr); V*sin(beta) + psi_dot*r_fr*sin(theta_fr)];
v_wheel_rl = [V*cos(beta) - psi_dot*r_rl*cos(theta_rl); V*sin(beta) - psi_dot*r_rl*sin(theta_rl)];
v_wheel_rr = [V*cos(beta) + psi_dot*r_rr*sin(theta_rr); V*sin(beta) - psi_dot*r_rr*cos(theta_rr)];
v_wheel = [v_wheel_fl(1) v_wheel_fl(2) v_wheel_fr(1) v_wheel_fr(2) v_wheel_rl(1) v_wheel_rl(2) v_wheel_rr(1) v_wheel_rr(2)]';

% wheel center speed
Vw_fl = norm(v_wheel_fl);
Vw_fr = norm(v_wheel_fr);
Vw_rl = norm(v_wheel_rl);
Vw_rr = norm(v_wheel_rr);

% wheel rotational velocities
v_rotation_fl = omega_wheel_fl*r_wheel;
v_rotation_fr = omega_wheel_fr*r_wheel;
v_rotation_rl = omega_wheel_rl*r_wheel;
v_rotation_rr = omega_wheel_rr*r_wheel;
v_rotation = [v_rotation_fl v_rotation_fr v_rotation_rl v_rotation_rr]';

% wheel slip angles
alpha_fl = delta - atan2(v_wheel_fl(2) , v_wheel_fl(1));
alpha_fr = delta - atan2(v_wheel_fr(2) , v_wheel_fr(1));
alpha_rl = -atan2(v_wheel_rl(2) , v_wheel_rl(1));
alpha_rr = -atan2(v_wheel_rr(2) , v_wheel_rr(1));
alpha = [alpha_fl alpha_fr alpha_rl alpha_rr]';

% wheel slip front left
[~, I] = min([(v_rotation_fl*cos(alpha_fl)) (Vw_fl)]);
if I==1
    %breaking or free rolling
    sigma_long_fl = (v_rotation_fl*cos(alpha_fl) - Vw_fl) / Vw_fl;
    sigma_lat_fl = (v_rotation_fl*sin(alpha_fl)) / (Vw_fl);
else
    %driving
    sigma_long_fl = (v_rotation_fl*cos(alpha_fl) - Vw_fl) / (v_rotation_fl*cos(alpha_fl));
    sigma_lat_fl = tan(alpha_fl);
end

% wheel slip front right
[~, I] = min([(v_rotation_fr*cos(alpha_fr)) (Vw_fr)]);
if I==1
    %breaking or free rolling
    sigma_long_fr = (v_rotation_fr*cos(alpha_fr) - Vw_fr) / Vw_fr;
    sigma_lat_fr = (v_rotation_fr*sin(alpha_fr)) / (Vw_fr);
else
    %driving
    sigma_long_fr = (v_rotation_fr*cos(alpha_fr) - Vw_fr) / (v_rotation_fr*cos(alpha_fr));
    sigma_lat_fr = tan(alpha_fr);
end

% wheel slip rear left
[~, I] = min([(v_rotation_rl*cos(alpha_rl)) (Vw_rl)]);
if I==1
    %breaking or free rolling
    sigma_long_rl = (v_rotation_rl*cos(alpha_rl) - Vw_rl) / Vw_rl;
    sigma_lat_rl = (v_rotation_rl*sin(alpha_rl)) / (Vw_rl);
else
    %driving
    sigma_long_rl = (v_rotation_rl*cos(alpha_rl) - Vw_rl) / (v_rotation_rl*cos(alpha_rl));
    sigma_lat_rl = tan(alpha_rl);
end

% wheel slip rear right
[~, I] = min([(v_rotation_rr*cos(alpha_rr)) (Vw_rr)]);
if I==1
    %breaking or free rolling
    sigma_long_rr = (v_rotation_rr*cos(alpha_rr) - Vw_rr) / Vw_rr;
    sigma_lat_rr = (v_rotation_rr*sin(alpha_rr)) / (Vw_rr);
else
    %driving
    sigma_long_rr = (v_rotation_rr*cos(alpha_rr) - Vw_rr) / (v_rotation_rr*cos(alpha_rr));
    sigma_lat_rr = tan(alpha_rr);
end

% make sure values are valid
if isnan(sigma_long_fl); sigma_long_fl = 0; end
if isnan(sigma_long_fr); sigma_long_fr = 0; end
if isnan(sigma_long_rl); sigma_long_rl = 0; end
if isnan(sigma_long_rr); sigma_long_rr = 0; end
if isnan(sigma_lat_fl); sigma_lat_fl = 0; end
if isnan(sigma_lat_fr); sigma_lat_fr = 0; end
if isnan(sigma_lat_rl); sigma_lat_rl = 0; end
if isnan(sigma_lat_rr); sigma_lat_rr = 0; end

if abs(sigma_long_fl) > 1; sigma_long_fl = 1*sign(sigma_long_fl); end
if abs(sigma_long_fr) > 1; sigma_long_fr = 1*sign(sigma_long_fr); end
if abs(sigma_long_rl) > 1; sigma_long_rl = 1*sign(sigma_long_rl); end
if abs(sigma_long_rr) > 1; sigma_long_rr = 1*sign(sigma_long_rr); end
if abs(sigma_lat_fl) > 1; sigma_lat_fl = 1*sign(sigma_lat_fl); end
if abs(sigma_lat_fr) > 1; sigma_lat_fr = 1*sign(sigma_lat_fr); end
if abs(sigma_lat_rl) > 1; sigma_lat_rl = 1*sign(sigma_lat_rl); end
if abs(sigma_lat_rr) > 1; sigma_lat_rr = 1*sign(sigma_lat_rr); end

% resultant slip
sigma_res_fl = sqrt(sigma_lat_fl^2 + sigma_long_fl^2);
sigma_res_fr = sqrt(sigma_lat_fr^2 + sigma_long_fr^2);
sigma_res_rl = sqrt(sigma_lat_rl^2 + sigma_long_rl^2);
sigma_res_rr = sqrt(sigma_lat_rr^2 + sigma_long_rr^2);

sigma_res  = [sigma_res_fl sigma_res_fr sigma_res_rl sigma_res_rr]';
sigma_long  = [sigma_long_fl sigma_long_fr sigma_long_rl sigma_long_rr]';
sigma_lat = [sigma_lat_fl sigma_lat_fr sigma_lat_rl sigma_lat_rr]';
